@@ -5,8 +5,6 @@ import os
 import re
 from collections import Counter
 from sklearn.model_selection import train_test_split
-from texta_mlp.mlp import MLP
-from texta_tools.embedding import W2VEmbedding
 from typing import List
 
 from .config import CRFConfig
@@ -32,8 +30,8 @@ class CRFExtractor:
 
     def __init__(self,
         description: str = "My CRF Extractor",
-        embedding: W2VEmbedding = None,
-        config: CRFConfig = CRFConfig()
+        config: CRFConfig = CRFConfig(),
+        embedding = None,
     ):
         self.description = description
         self.config = config
@@ -45,8 +43,7 @@ class CRFExtractor:
         return self.description
 
 
-    @staticmethod
-    def _validate_and_extract_layers(mlp_document: dict, field_name: str):
+    def _validate_and_extract_layers(self, mlp_document: dict, field_name: str):
         """
         Validates the MLP document structure to have proper fields.
         """
@@ -61,13 +58,14 @@ class CRFExtractor:
         # check the resulting subdocument structure
         if not isinstance(mlp_subdoc, dict):
             raise exceptions.InvalidInputError("Document is not a dict!")
-        if field_name not in mlp_subdoc:
-            raise exceptions.InvalidInputError(f"Field '{field_name}' not present in the document!")
         # empty dict for layers
         layers = {}
+
+        fields_to_check = set(self.config.feature_layers + self.config.context_feature_layers)
+
         # check if mlp fields are present
         for field in CRFExtractor.MLP_FIELDS:
-            if field not in mlp_subdoc:
+            if field not in mlp_subdoc and field in fields_to_check:
                 raise exceptions.InvalidInputError(f"Field '{field}' not present in the document!")
             # add layer if not present
             if field not in layers:
@@ -85,7 +83,7 @@ class CRFExtractor:
         return layers
 
 
-    def _parse_mlp_document(self, mlp_document: dict, add_labels: bool = True, mlp_field: str = "text"):
+    def _parse_mlp_document(self, mlp_document: dict, add_labels: bool = True, mlp_field: str = "text_mlp"):
         """
         Parses MLP output document. Extracts tokens, lemmas, and POS tags.
         Adds labels from texta_facts.
@@ -110,15 +108,19 @@ class CRFExtractor:
                             # replace initial labels with correct ones
                             for j in range(num_tokens_before_match, num_tokens_before_match+num_tokens_match):
                                 labels[i][j] = label
-
-                yield [(token, layers[FEATURE_LAYER_LEMMA][i][j], layers[FEATURE_LAYER_POS][i][j], labels[i][j]) for j, token in enumerate(sentence)]
+                try:
+                    yield [(token, layers[FEATURE_LAYER_LEMMA][i][j], layers[FEATURE_LAYER_POS][i][j], labels[i][j]) for j, token in enumerate(sentence)]
+                except IndexError:
+                    raise exceptions.InvalidInputError("Layer lengths do not match!")
         else:
             for i, sentence in enumerate(layers[FEATURE_LAYER_WORD]):
-                yield i, [(token, layers[FEATURE_LAYER_LEMMA][i][j], layers[FEATURE_LAYER_POS][i][j]) for j, token in enumerate(sentence)]
+                try:
+                    yield i, [(token, layers[FEATURE_LAYER_LEMMA][i][j], layers[FEATURE_LAYER_POS][i][j]) for j, token in enumerate(sentence)]
+                except IndexError:
+                    raise exceptions.InvalidInputError("Layer lengths do not match!")
 
 
-
-    def train(self, mlp_documents: List[dict], mlp_field: str = "text", save_path: str = "my_crf_model"):
+    def train(self, mlp_documents: List[dict], mlp_field: str = "text_mlp", save_path: str = "my_crf_model"):
         """
         Trains & saves the model.
         Model has to be saved by the crfsuite package because it contains C++ bindings.
@@ -164,12 +166,15 @@ class CRFExtractor:
 
 
     @check_model_loaded
-    def tag(self, mlp_document: dict, field_name: str = "text"):
+    def tag(self, mlp_document: dict, field_name: str = "text_mlp"):
         """
         Tags input MLP document.
         """
-        output = []
-        seqs_to_predict = self._parse_mlp_document(mlp_document, add_labels=False)
+        facts = []
+        seqs_to_predict = self._parse_mlp_document(mlp_document, add_labels=False, mlp_field=field_name)
+        tokenized_text = mlp_document[field_name][FEATURE_LAYER_WORD]
+        if field_name not in mlp_document:
+            raise exceptions.InvalidInputError(f"Field {field_name} not present in input document!")
         sentences = [sent.split(" ") for sent in mlp_document[field_name][FEATURE_LAYER_WORD].split(" \n ")]
         # predict on each sentence
         for i, seq_to_predict in seqs_to_predict:
@@ -177,21 +182,35 @@ class CRFExtractor:
             # predict
             result = self.model.tag(features_to_predict)
             # generate text tokens for final output
-            for tag in self._process_tag_output(result, sentences, i):
-                output.append(tag)
-        return output
+            for tag in self._process_tag_output(result, sentences, i, field_name):
+                facts.append(tag)
+
+        out_doc = {
+            "text": tokenized_text,
+            "texta_facts": facts
+        }
+        return out_doc
 
 
     @check_model_loaded
     def get_features(self, n=20):
         info = self.model.info()
+        all_feats = Counter(info.state_features).most_common()
+
+        positive_features = [{"feature": f[0], "coefficient": c, "label": f[1]} for f, c in Counter(info.state_features).most_common(n)]
+        negative_features = [{"feature": f[0], "coefficient": c, "label": f[1]} for f, c in all_feats[-n:]]
+
         return {
-            "positive": Counter(info.state_features).most_common(n),
-            "negative": Counter(info.state_features).most_common()[-n:]
+            "features": {
+                "positive": positive_features,
+                "negative": negative_features
+            },
+            "total_features": len(list(all_feats)),
+            "showing_features": len(positive_features+negative_features)
         }
 
 
-    def _process_tag_output(self, tokens: List[str], sentences: List[List[str]], sent_index: int):
+    def _process_tag_output(self, tokens: List[str], sentences: List[List[str]], sent_index: int, field_name: str):
         """
         Translates result tokens into entities.
         """
@@ -220,6 +239,7 @@ class CRFExtractor:
                 "fact": fact_name,
                 "str_val": str_val,
                 "sent_index": sent_index,
+                "doc_path": field_name,
                 "spans": json.dumps(matching_spans)
             }
             yield fact
