@@ -1,4 +1,3 @@
-from texta_tools.embedding import W2VEmbedding
 from typing import List, Tuple
 from . import exceptions
 
@@ -23,7 +22,7 @@ def has_digit(string: str):
     return any(chr.isdigit() for chr in string)
 
 
-def extract_features(item: dict, layers_to_use: List[str], extractors_to_use: List[str], word_index: int, suffix_len: Tuple[int], embedding: W2VEmbedding):
+def extract_features(item: dict, layers_to_use: List[str], extractors_to_use: List[str], word_index: int, suffix_len: Tuple[int], embedding):
     """
     Extract features from token and surrounding tokens based on window size.
     """
@@ -52,7 +51,7 @@ def extract_features(item: dict, layers_to_use: List[str], extractors_to_use: Li
             features.append(f"{word_index}:{FEATURE_LAYER_WORD}.{extractor}={feature_value}")
         # add more token-based extractors here!
     # get similar lemmas from embedding
-    if embedding:
+    if embedding and FEATURE_LAYER_LEMMA in item:
         most_similar_lemmas = embedding.get_similar([item[FEATURE_LAYER_LEMMA]], n=1)
         if len(most_similar_lemmas):
             most_similar_lemma = most_similar_lemmas[0]["phrase"]
@@ -73,21 +72,26 @@ def word2features(sent: list, i: int, config, embedding):
     if config.bias:
         features.append("bias")
     # this is the current token
-    item = {
-        FEATURE_LAYER_WORD: sent[i][0],
-        FEATURE_LAYER_LEMMA: sent[i][1],
-        FEATURE_LAYER_POS: sent[i][2]
-    }    
+    item = {}
+    if FEATURE_LAYER_WORD in config.feature_layers:
+        item[FEATURE_LAYER_WORD] = sent[i][0]
+    if FEATURE_LAYER_LEMMA in config.feature_layers:
+        item[FEATURE_LAYER_LEMMA] = sent[i][1]
+    if FEATURE_LAYER_POS in config.feature_layers:
+        item[FEATURE_LAYER_POS] = sent[i][2]
+
     features.extend(extract_features(item, config.feature_layers, config.feature_extractors, "0", config.suffix_len, embedding))
     # check if not the first token in sentence
     if i > 0:
         for window in range(1, config.window_size+1):
             try:
-                item1 = {
-                    FEATURE_LAYER_WORD: sent[i-window][0],
-                    FEATURE_LAYER_LEMMA: sent[i-window][1],
-                    FEATURE_LAYER_POS: sent[i-window][2]
-                }
+                item1 = {}
+                if FEATURE_LAYER_WORD in config.feature_context_layers:
+                    item1[FEATURE_LAYER_WORD] = sent[i-window][0]
+                if FEATURE_LAYER_LEMMA in config.feature_context_layers:
+                    item1[FEATURE_LAYER_LEMMA] = sent[i-window][1]
+                if FEATURE_LAYER_POS in config.feature_context_layers:
+                    item1[FEATURE_LAYER_POS] = sent[i-window][2]
                 features.extend(extract_features(item1, config.context_feature_layers, config.context_feature_extractors, f"-{window}", config.suffix_len, embedding))
             except:
                 IndexError
@@ -98,11 +102,13 @@ def word2features(sent: list, i: int, config, embedding):
     if i < len(sent)-1:
         for window in range(1, config.window_size+1):
             try:
-                item1 = {
-                    FEATURE_LAYER_WORD: sent[i+window][0],
-                    FEATURE_LAYER_LEMMA: sent[i+window][1],
-                    FEATURE_LAYER_POS: sent[i+window][2]
-                }
+                item1 = {}
+                if FEATURE_LAYER_WORD in config.feature_context_layers:
+                    item1[FEATURE_LAYER_WORD] = sent[i+window][0]
+                if FEATURE_LAYER_LEMMA in config.feature_context_layers:
+                    item1[FEATURE_LAYER_LEMMA] = sent[i+window][1]
+                if FEATURE_LAYER_POS in config.feature_context_layers:
+                    item1[FEATURE_LAYER_POS] = sent[i+window][2]
                 features.extend(extract_features(item1, config.context_feature_layers, config.context_feature_extractors, f"+{window}", config.suffix_len, embedding))
             except:
                 IndexError
